@@ -13,17 +13,25 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <kurldrag.h>
+#include "collectionlist.h"
+
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
-#include <kpopupmenu.h>
-#include <kiconloader.h>
+#include <kmenu.h>
 #include <kconfig.h>
-#include <kaction.h>
-#include <kurl.h>
+#include <kconfiggroup.h>
+#include <kactioncollection.h>
+#include <ktoolbarpopupaction.h>
+#include <kdirwatch.h>
+
+#include <QList>
+#include <QDragMoveEvent>
+#include <QDropEvent>
+#include <QApplication>
+#include <QClipboard>
+#include <QFileInfo>
 
-#include "collectionlist.h"
 #include "playlistcollection.h"
 #include "splashscreen.h"
 #include "stringshare.h"
@@ -34,7 +42,7 @@
 #include "coverinfo.h"
 #include "covermanager.h"
 
-using namespace ActionCollection;
+using ActionCollection::action;
 
 ////////////////////////////////////////////////////////////////////////////////
 // static methods
@@ -50,7 +58,7 @@ CollectionList *CollectionList::instance()
 void CollectionList::initialize(PlaylistCollection *collection)
 {
     if(m_list)
-	return;
+        return;
 
     // We have to delay initilaization here because dynamic_cast or comparing to
     // the collection instance won't work in the PlaylistBox::Item initialization
@@ -61,45 +69,45 @@ void CollectionList::initialize(PlaylistCollection *collection)
 
     FileHandleHash::Iterator end = Cache::instance()->end();
     for(FileHandleHash::Iterator it = Cache::instance()->begin(); it != end; ++it)
-	new CollectionListItem(*it);
+        new CollectionListItem(*it);
 
     SplashScreen::update();
 
     // The CollectionList is created with sorting disabled for speed.  Re-enable
     // it here, and perform the sort.
     KConfigGroup config(KGlobal::config(), "Playlists");
-    
-    SortOrder order = Descending;
-    if(config.readBoolEntry("CollectionListSortAscending", true))
-	order = Ascending;
+
+    Qt::SortOrder order = Qt::DescendingOrder;
+    if(config.readEntry("CollectionListSortAscending", true))
+        order = Qt::AscendingOrder;
 
     m_list->setSortOrder(order);
-    m_list->setSortColumn(config.readNumEntry("CollectionListSortColumn", 1));
+    m_list->setSortColumn(config.readEntry("CollectionListSortColumn", 1));
 
     m_list->sort();
 
-    collection->setupPlaylist(m_list, "folder_sound");
+    collection->setupPlaylist(m_list, "folder-sound");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // public methods
 ////////////////////////////////////////////////////////////////////////////////
 
-PlaylistItem *CollectionList::createItem(const FileHandle &file, QListViewItem *, bool)
+PlaylistItem *CollectionList::createItem(const FileHandle &file, Q3ListViewItem *, bool)
 {
     // It's probably possible to optimize the line below away, but, well, right
     // now it's more important to not load duplicate items.
 
-    if(m_itemsDict.find(file.absFilePath()))
-	return 0;
+    if(m_itemsDict.contains(file.absFilePath()))
+        return 0;
 
     PlaylistItem *item = new CollectionListItem(file);
-    
+
     if(!item->isValid()) {
-	kdError() << "CollectionList::createItem() -- A valid tag was not created for \""
-		  << file.absFilePath() << "\"" << endl;
-	delete item;
-	return 0;
+        kError() << "CollectionList::createItem() -- A valid tag was not created for \""
+                  << file.absFilePath() << "\"" << endl;
+        delete item;
+        return 0;
     }
 
     setupItem(item);
@@ -110,8 +118,8 @@ PlaylistItem *CollectionList::createItem(const FileHandle &file, QListViewItem *
 void CollectionList::clearItems(const PlaylistItemList &items)
 {
     for(PlaylistItemList::ConstIterator it = items.begin(); it != items.end(); ++it) {
-	Cache::instance()->remove((*it)->file());
-	clearItem(*it, false);
+        Cache::instance()->remove((*it)->file());
+        clearItem(*it, false);
     }
 
     dataChanged();
@@ -121,78 +129,78 @@ void CollectionList::setupTreeViewEntries(ViewMode *viewMode) const
 {
     TreeViewMode *treeViewMode = dynamic_cast<TreeViewMode *>(viewMode);
     if(!treeViewMode) {
-	kdWarning(65432) << "Can't setup entries on a non-tree-view mode!\n";
-	return;
+        kWarning(65432) << "Can't setup entries on a non-tree-view mode!\n";
+        return;
     }
 
-    QValueList<int> columnList;
+    QList<int> columnList;
     columnList << PlaylistItem::ArtistColumn;
     columnList << PlaylistItem::GenreColumn;
     columnList << PlaylistItem::AlbumColumn;
 
-    QStringList items;
-    for(QValueList<int>::Iterator colIt = columnList.begin(); colIt != columnList.end(); ++colIt) {
-	items.clear();
-	for(TagCountDictIterator it(*m_columnTags[*colIt]); it.current(); ++it)
-	    items << it.currentKey();
-
-	treeViewMode->addItems(items, *colIt);
-    }
+    foreach(int column, columnList)
+        treeViewMode->addItems(m_columnTags[column]->keys(), column);
 }
 
 void CollectionList::slotNewItems(const KFileItemList &items)
 {
     QStringList files;
 
-    for(KFileItemListIterator it(items); it.current(); ++it)
-	files.append((*it)->url().path());
+    for(KFileItemList::ConstIterator it = items.begin(); it != items.end(); ++it)
+        files.append((*it).url().path());
 
     addFiles(files);
     update();
 }
 
-void CollectionList::slotRefreshItems(const KFileItemList &items)
+void CollectionList::slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > &items)
 {
-    for(KFileItemListIterator it(items); it.current(); ++it) {
-	CollectionListItem *item = lookup((*it)->url().path());
+    for(int i = 0; i < items.count(); ++i) {
+        const KFileItem fileItem = items[i].second;
+        CollectionListItem *item = lookup(fileItem.url().path());
 
-	if(item) {
-	    item->refreshFromDisk();
+        if(item) {
+            item->refreshFromDisk();
 
-	    // If the item is no longer on disk, remove it from the collection.
+            // If the item is no longer on disk, remove it from the collection.
 
-	    if(item->file().fileInfo().exists())
-		item->repaint();
-	    else
-		delete item;
-	}
+            if(item->file().fileInfo().exists())
+                item->repaint();
+            else
+                delete item;
+        }
     }
 
     update();
 }
 
-void CollectionList::slotDeleteItem(KFileItem *item)
+void CollectionList::slotDeleteItem(const KFileItem &item)
 {
-    CollectionListItem *listItem = lookup(item->url().path());
+    CollectionListItem *listItem = lookup(item.url().path());
     if(listItem)
-	clearItem(listItem);
+        clearItem(listItem);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // public slots
 ////////////////////////////////////////////////////////////////////////////////
 
+void CollectionList::paste()
+{
+    decode(QApplication::clipboard()->mimeData());
+}
+
 void CollectionList::clear()
 {
-    int result = KMessageBox::warningContinueCancel(this, 
-	i18n("Removing an item from the collection will also remove it from "
-	     "all of your playlists. Are you sure you want to continue?\n\n"
-	     "Note, however, that if the directory that these files are in is in "
-	     "your \"scan on startup\" list, they will be readded on startup."));
+    int result = KMessageBox::warningContinueCancel(this,
+        i18n("Removing an item from the collection will also remove it from "
+             "all of your playlists. Are you sure you want to continue?\n\n"
+             "Note, however, that if the directory that these files are in is in "
+             "your \"scan on startup\" list, they will be readded on startup."));
 
     if(result == KMessageBox::Continue) {
-	Playlist::clear();
-	emit signalCollectionChanged();
+        Playlist::clear();
+        emit signalCollectionChanged();
     }
 }
 
@@ -200,10 +208,10 @@ void CollectionList::slotCheckCache()
 {
     PlaylistItemList invalidItems;
 
-    for(QDictIterator<CollectionListItem>it(m_itemsDict); it.current(); ++it) {
-	if(!it.current()->checkCurrent())
-	    invalidItems.append(*it);
-	processEvents();
+    foreach(CollectionListItem *item, m_itemsDict) {
+        if(!item->checkCurrent())
+            invalidItems.append(item);
+        processEvents();
     }
 
     clearItems(invalidItems);
@@ -216,7 +224,8 @@ void CollectionList::slotRemoveItem(const QString &file)
 
 void CollectionList::slotRefreshItem(const QString &file)
 {
-    m_itemsDict[file]->refresh();
+    if(m_itemsDict[file])
+        m_itemsDict[file]->refresh();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -225,73 +234,65 @@ void CollectionList::slotRefreshItem(const QString &file)
 
 CollectionList::CollectionList(PlaylistCollection *collection) :
     Playlist(collection, true),
-    m_itemsDict(5003),
     m_columnTags(15, 0)
 {
-    new KAction(i18n("Show Playing"), KShortcut(), actions(), "showPlaying");
+    QAction *spaction = ActionCollection::actions()->addAction("showPlaying");
+    spaction->setText(i18n("Show Playing"));
+    connect(spaction, SIGNAL(triggered(bool) ), SLOT(slotShowPlaying()));
 
-    connect(action("showPlaying"), SIGNAL(activated()), this, SLOT(slotShowPlaying()));
-
-    connect(action<KToolBarPopupAction>("back")->popupMenu(), SIGNAL(aboutToShow()),
-	    this, SLOT(slotPopulateBackMenu()));
-    connect(action<KToolBarPopupAction>("back")->popupMenu(), SIGNAL(activated(int)),
-	    this, SLOT(slotPlayFromBackMenu(int)));
+    connect(action<KToolBarPopupAction>("back")->menu(), SIGNAL(aboutToShow()),
+            this, SLOT(slotPopulateBackMenu()));
+    connect(action<KToolBarPopupAction>("back")->menu(), SIGNAL(triggered(QAction *)),
+            this, SLOT(slotPlayFromBackMenu(QAction *)));
     setSorting(-1); // Temporarily disable sorting to add items faster.
 
-    m_columnTags[PlaylistItem::ArtistColumn] = new TagCountDict(5001, false);
-    m_columnTags[PlaylistItem::ArtistColumn]->setAutoDelete(true);
-
-    m_columnTags[PlaylistItem::AlbumColumn] = new TagCountDict(5001, false);
-    m_columnTags[PlaylistItem::AlbumColumn]->setAutoDelete(true);
-
-    m_columnTags[PlaylistItem::GenreColumn] = new TagCountDict(5001, false);
-    m_columnTags[PlaylistItem::GenreColumn]->setAutoDelete(true);
-
-    polish();
+    m_columnTags[PlaylistItem::ArtistColumn] = new TagCountDict;
+    m_columnTags[PlaylistItem::AlbumColumn] = new TagCountDict;
+    m_columnTags[PlaylistItem::GenreColumn] = new TagCountDict;
 }
 
 CollectionList::~CollectionList()
 {
     KConfigGroup config(KGlobal::config(), "Playlists");
     config.writeEntry("CollectionListSortColumn", sortColumn());
-    config.writeEntry("CollectionListSortAscending", sortOrder() == Ascending);
+    config.writeEntry("CollectionListSortAscending", sortOrder() == Qt::AscendingOrder);
 
     // The CollectionListItems will try to remove themselves from the
     // m_columnTags member, so we must make sure they're gone before we
     // are.
 
     clearItems(items());
-    for(TagCountDicts::Iterator it = m_columnTags.begin(); it != m_columnTags.end(); ++it)
-	delete *it;
+
+    qDeleteAll(m_columnTags);
+    m_columnTags.clear();
 }
 
 void CollectionList::contentsDropEvent(QDropEvent *e)
 {
     if(e->source() == this)
-	return; // Don't rearrange in the CollectionList.
-    else 
-	Playlist::contentsDropEvent(e);
+        return; // Don't rearrange in the CollectionList.
+    else
+        Playlist::contentsDropEvent(e);
 }
 
 void CollectionList::contentsDragMoveEvent(QDragMoveEvent *e)
 {
-    if(canDecode(e) && e->source() != this)
-	e->accept(true);
+    if(e->source() != this)
+        Playlist::contentsDragMoveEvent(e);
     else
-	e->accept(false);
+        e->setAccepted(false);
 }
 
-QString CollectionList::addStringToDict(const QString &value, unsigned column)
+QString CollectionList::addStringToDict(const QString &value, int column)
 {
-    if(column > m_columnTags.count() || value.stripWhiteSpace().isEmpty())
-	return QString::null;
+    if(column > m_columnTags.count() || value.trimmed().isEmpty())
+        return QString();
 
-    int *refCountPtr = m_columnTags[column]->find(value);
-    if(refCountPtr)
-	++(*refCountPtr);
+    if(m_columnTags[column]->contains(value))
+        ++((*m_columnTags[column])[value]);
     else {
-	m_columnTags[column]->insert(value, new int(1));
-	emit signalNewTag(value, column);
+        m_columnTags[column]->insert(value, 1);
+        emit signalNewTag(value, column);
     }
 
     return value;
@@ -306,46 +307,50 @@ QStringList CollectionList::uniqueSet(UniqueSetType t) const
     case Artists:
         column = PlaylistItem::ArtistColumn;
     break;
-    
+
     case Albums:
         column = PlaylistItem::AlbumColumn;
     break;
-    
+
     case Genres:
         column = PlaylistItem::GenreColumn;
     break;
 
     default:
-	return QStringList();
+        return QStringList();
     }
 
-    if((unsigned) column >= m_columnTags.count())
-	return QStringList();
-
-    TagCountDictIterator it(*m_columnTags[column]);
-    QStringList list;
-
-    for(; it.current(); ++it)
-	list += it.currentKey();
-
-    return list;
+    return m_columnTags[column]->keys();
 }
 
-void CollectionList::removeStringFromDict(const QString &value, unsigned column)
+CollectionListItem *CollectionList::lookup(const QString &file) const
 {
-    if(column > m_columnTags.count() || value.isEmpty())
-	return;
+    return m_itemsDict.value(file, 0);
+}
 
-    int *refCountPtr = m_columnTags[column]->find(value);
-    if(refCountPtr) {
-	--(*refCountPtr);
-	if(*refCountPtr == 0) {
-	    emit signalRemovedTag(value, column);
-	    m_columnTags[column]->remove(value);
-	}
+void CollectionList::removeStringFromDict(const QString &value, int column)
+{
+    if(column > m_columnTags.count() || value.trimmed().isEmpty())
+        return;
+
+    if(m_columnTags[column]->contains(value) &&
+       --((*m_columnTags[column])[value])) // If the decrement goes to 0...
+    {
+        emit signalRemovedTag(value, column);
+        m_columnTags[column]->remove(value);
     }
 }
 
+void CollectionList::addWatched(const QString &file)
+{
+    m_dirWatch->addFile(file);
+}
+
+void CollectionList::removeWatched(const QString &file)
+{
+    m_dirWatch->removeFile(file);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // CollectionListItem public methods
 ////////////////////////////////////////////////////////////////////////////////
@@ -359,46 +364,46 @@ void CollectionListItem::refresh()
     data()->cachedWidths.resize(columns);
 
     for(int i = offset; i < columns; i++) {
-	int id = i - offset;
-	if(id != TrackNumberColumn && id != LengthColumn) {        
-	    // All columns other than track num and length need local-encoded data for sorting        
+        int id = i - offset;
+        if(id != TrackNumberColumn && id != LengthColumn) {
+            // All columns other than track num and length need local-encoded data for sorting
 
-	    QCString lower = text(i).lower().local8Bit();
+            QByteArray toLower = text(i).toLower().toLocal8Bit();
 
-	    // For some columns, we may be able to share some strings
+            // For some columns, we may be able to share some strings
 
-	    if((id == ArtistColumn) || (id == AlbumColumn) ||
-	       (id == GenreColumn)  || (id == YearColumn)  ||
-	       (id == CommentColumn))
-	    {
-		lower = StringShare::tryShare(lower);
+            if((id == ArtistColumn) || (id == AlbumColumn) ||
+               (id == GenreColumn)  || (id == YearColumn)  ||
+               (id == CommentColumn))
+            {
+                toLower = StringShare::tryShare(toLower);
 
-		if(id != YearColumn && id != CommentColumn && data()->local8Bit[id] != lower) {
-		    CollectionList::instance()->removeStringFromDict(data()->local8Bit[id], id);
-		    CollectionList::instance()->addStringToDict(text(i), id);
-		}
-	    }
+                if(id != YearColumn && id != CommentColumn && data()->local8Bit[id] != toLower) {
+                    CollectionList::instance()->removeStringFromDict(data()->local8Bit[id], id);
+                    CollectionList::instance()->addStringToDict(text(i), id);
+                }
+            }
 
-	    data()->local8Bit[id] = lower;
-	}
+            data()->local8Bit[id] = toLower;
+        }
 
-	int newWidth = width(listView()->fontMetrics(), listView(), i);
-	data()->cachedWidths[i] = newWidth;
+        int newWidth = width(listView()->fontMetrics(), listView(), i);
+        data()->cachedWidths[i] = newWidth;
 
-	if(newWidth != data()->cachedWidths[i])
-	    playlist()->slotWeightDirty(i);
+        if(newWidth != data()->cachedWidths[i])
+            playlist()->slotWeightDirty(i);
     }
 
     file().coverInfo()->setCover();
 
     if(listView()->isVisible())
-	repaint();
+        repaint();
 
     for(PlaylistItemList::Iterator it = m_children.begin(); it != m_children.end(); ++it) {
-	(*it)->playlist()->update();
-	(*it)->playlist()->dataChanged();
-	if((*it)->listView()->isVisible())
-	    (*it)->repaint();
+        (*it)->playlist()->update();
+        (*it)->playlist()->dataChanged();
+        if((*it)->listView()->isVisible())
+            (*it)->repaint();
     }
 
     CollectionList::instance()->dataChanged();
@@ -408,12 +413,12 @@ void CollectionListItem::refresh()
 PlaylistItem *CollectionListItem::itemForPlaylist(const Playlist *playlist)
 {
     if(playlist == CollectionList::instance())
-	return this;
+        return this;
 
     PlaylistItemList::ConstIterator it;
     for(it = m_children.begin(); it != m_children.end(); ++it)
-	if((*it)->playlist() == playlist)
-	    return *it;
+        if((*it)->playlist() == playlist)
+            return *it;
     return 0;
 }
 
@@ -422,7 +427,7 @@ void CollectionListItem::updateCollectionDict(const QString &oldPath, const QStr
     CollectionList *collection = CollectionList::instance();
 
     if(!collection)
-	return;
+        return;
 
     collection->removeFromDict(oldPath);
     collection->addToDict(newPath, this);
@@ -430,9 +435,9 @@ void CollectionListItem::updateCollectionDict(const QString &oldPath, const QStr
 
 void CollectionListItem::repaint() const
 {
-    QListViewItem::repaint();
+    Q3ListViewItem::repaint();
     for(PlaylistItemList::ConstIterator it = m_children.begin(); it != m_children.end(); ++it)
-	(*it)->repaint();
+        (*it)->repaint();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -445,21 +450,21 @@ CollectionListItem::CollectionListItem(const FileHandle &file) :
 {
     CollectionList *l = CollectionList::instance();
     if(l) {
-	l->addToDict(file.absFilePath(), this);
+        l->addToDict(file.absFilePath(), this);
 
-	data()->fileHandle = file;
+        data()->fileHandle = file;
 
-	if(file.tag()) {
-	    refresh();
-	    l->dataChanged();
-	    // l->addWatched(m_path);
-	}
-	else
-	    kdError() << "CollectionListItem::CollectionListItem() -- Tag() could not be created." << endl;
+        if(file.tag()) {
+            refresh();
+            l->dataChanged();
+            // l->addWatched(m_path);
+        }
+        else
+            kError() << "CollectionListItem::CollectionListItem() -- Tag() could not be created." << endl;
     }
     else
-	kdError(65432) << "CollectionListItems should not be created before "
-		       << "CollectionList::initialize() has been called." << endl;
+        kError(65432) << "CollectionListItems should not be created before "
+                       << "CollectionList::initialize() has been called." << endl;
 
     SplashScreen::increment();
 }
@@ -469,18 +474,18 @@ CollectionListItem::~CollectionListItem()
     m_shuttingDown = true;
 
     for(PlaylistItemList::ConstIterator it = m_children.begin();
-	it != m_children.end();
-	++it)
+        it != m_children.end();
+        ++it)
     {
-	delete *it;
+        delete *it;
     }
 
     CollectionList *l = CollectionList::instance();
     if(l) {
-	l->removeFromDict(file().absFilePath());
-	l->removeStringFromDict(file().tag()->album(), AlbumColumn);
-	l->removeStringFromDict(file().tag()->artist(), ArtistColumn);
-	l->removeStringFromDict(file().tag()->genre(), GenreColumn);
+        l->removeFromDict(file().absFilePath());
+        l->removeStringFromDict(file().tag()->album(), AlbumColumn);
+        l->removeStringFromDict(file().tag()->artist(), ArtistColumn);
+        l->removeStringFromDict(file().tag()->genre(), GenreColumn);
     }
 }
 
@@ -492,20 +497,22 @@ void CollectionListItem::addChildItem(PlaylistItem *child)
 void CollectionListItem::removeChildItem(PlaylistItem *child)
 {
     if(!m_shuttingDown)
-	m_children.remove(child);
+        m_children.removeAll(child);
 }
 
 bool CollectionListItem::checkCurrent()
 {
     if(!file().fileInfo().exists() || !file().fileInfo().isFile())
-	return false;
+        return false;
 
     if(!file().current()) {
-	file().refresh();
-	refresh();
+        file().refresh();
+        refresh();
     }
 
     return true;
 }
 
 #include "collectionlist.moc"
+
+// vim: set et sw=4 tw=0 sta:
