@@ -13,26 +13,28 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <config.h>
+#include "playlistitem.h"
 
+#include <config-juk.h>
 #include <kdebug.h>
-#include <kaction.h>
 #include <kiconloader.h>
 
-#include "playlistitem.h"
+#include <QPixmap>
+#include <QFileInfo>
+
 #include "collectionlist.h"
 #include "musicbrainzquery.h"
 #include "tag.h"
-#include "actioncollection.h"
-#include "ktrm.h"
 #include "coverinfo.h"
+#include "covermanager.h"
 #include "tagtransactionmanager.h"
 
 PlaylistItemList PlaylistItem::m_playingItems; // static
 
 static void startMusicBrainzQuery(const FileHandle &file)
 {
-#if HAVE_MUSICBRAINZ
+#ifdef HAVE_TUNEPIMP
+    // This deletes itself when finished.
     new MusicBrainzLookup(file);
 #else
     Q_UNUSED(file)
@@ -52,14 +54,14 @@ PlaylistItem::~PlaylistItem()
 
     m_collectionItem->removeChildItem(this);
 
-    if(m_playingItems.find(this) != m_playingItems.end()) {
-	m_playingItems.remove(this);
-	if(m_playingItems.isEmpty())
-	    playlist()->setPlaying(0);
+    if(m_playingItems.contains(this)) {
+        m_playingItems.removeAll(this);
+        if(m_playingItems.isEmpty())
+            playlist()->setPlaying(0);
     }
 
     if(m_watched)
-	Pointer::clear(this);
+        Pointer::clear(this);
 }
 
 void PlaylistItem::setFile(const FileHandle &file)
@@ -84,59 +86,66 @@ FileHandle PlaylistItem::file() const
 
 const QPixmap *PlaylistItem::pixmap(int column) const
 {
-    static QPixmap image(SmallIcon("image"));
+    static QPixmap image(SmallIcon("image-x-generic"));
     static QPixmap playing(UserIcon("playing"));
 
     int offset = playlist()->columnOffset();
 
-    if((column - offset) == CoverColumn && d->fileHandle.coverInfo()->hasCover())
+    // Don't use hasCover here because that may dig into the track itself.
+    // Besides, we really just want to know if the cover manager has a cover
+    // for the track.
+
+    if((column - offset) == CoverColumn &&
+        d->fileHandle.coverInfo()->coverId() != CoverManager::NoMatch)
+    {
         return &image;
+    }
 
     if(column == playlist()->leftColumn() &&
        m_playingItems.contains(const_cast<PlaylistItem *>(this)))
-	return &playing;
+        return &playing;
 
-    return KListViewItem::pixmap(column);
+    return K3ListViewItem::pixmap(column);
 }
 
 QString PlaylistItem::text(int column) const
 {
     if(!d->fileHandle.tag())
-	return QString::null;
+        return QString();
 
     int offset = playlist()->columnOffset();
 
     switch(column - offset) {
     case TrackColumn:
-	return d->fileHandle.tag()->title();
+        return d->fileHandle.tag()->title();
     case ArtistColumn:
-	return d->fileHandle.tag()->artist();
+        return d->fileHandle.tag()->artist();
     case AlbumColumn:
-	return d->fileHandle.tag()->album();
+        return d->fileHandle.tag()->album();
     case CoverColumn:
-	return QString::null;
+        return QString();
     case TrackNumberColumn:
-	return d->fileHandle.tag()->track() > 0
-	    ? QString::number(d->fileHandle.tag()->track())
-	    : QString::null;
+        return d->fileHandle.tag()->track() > 0
+            ? QString::number(d->fileHandle.tag()->track())
+            : QString();
     case GenreColumn:
-	return d->fileHandle.tag()->genre();
+        return d->fileHandle.tag()->genre();
     case YearColumn:
-	return d->fileHandle.tag()->year() > 0 
-	    ? QString::number(d->fileHandle.tag()->year())
-	    : QString::null;
+        return d->fileHandle.tag()->year() > 0
+            ? QString::number(d->fileHandle.tag()->year())
+            : QString();
     case LengthColumn:
-	return d->fileHandle.tag()->lengthString();
+        return d->fileHandle.tag()->lengthString();
     case BitrateColumn:
-	return QString::number(d->fileHandle.tag()->bitrate());
+        return QString::number(d->fileHandle.tag()->bitrate());
     case CommentColumn:
-	return d->fileHandle.tag()->comment();
+        return d->fileHandle.tag()->comment();
     case FileNameColumn:
-	return d->fileHandle.fileInfo().fileName();
+        return d->fileHandle.fileInfo().fileName();
     case FullPathColumn:
-	return d->fileHandle.fileInfo().absFilePath();
+        return d->fileHandle.fileInfo().absoluteFilePath();
     default:
-	return KListViewItem::text(column);
+        return K3ListViewItem::text(column);
     }
 }
 
@@ -144,31 +153,31 @@ void PlaylistItem::setText(int column, const QString &text)
 {
     int offset = playlist()->columnOffset();
     if(column - offset >= 0 && column + offset <= lastColumn()) {
-	KListViewItem::setText(column, QString::null);
-	return;
+        K3ListViewItem::setText(column, QString());
+        return;
     }
 
-    KListViewItem::setText(column, text);
+    K3ListViewItem::setText(column, text);
     playlist()->slotWeightDirty(column);
 }
 
 void PlaylistItem::setPlaying(bool playing, bool master)
 {
-    m_playingItems.remove(this);
+    m_playingItems.removeAll(this);
 
     if(playing) {
-	if(master)
-	    m_playingItems.prepend(this);
-	else
-	    m_playingItems.append(this);
+        if(master)
+            m_playingItems.prepend(this);
+        else
+            m_playingItems.append(this);
     }
     else {
 
-	// This is a tricky little recursion, but it
-	// in fact does clear the list.
+        // This is a tricky little recursion, but it
+        // in fact does clear the list.
 
-	if(!m_playingItems.isEmpty())
-	    m_playingItems.front()->setPlaying(false);
+        if(!m_playingItems.isEmpty())
+            m_playingItems.front()->setPlaying(false);
     }
 
     listView()->triggerUpdate();
@@ -177,7 +186,7 @@ void PlaylistItem::setPlaying(bool playing, bool master)
 void PlaylistItem::setSelected(bool selected)
 {
     playlist()->markItemSelected(this, selected);
-    KListViewItem::setSelected(selected);
+    K3ListViewItem::setSelected(selected);
 }
 
 void PlaylistItem::guessTagInfo(TagGuesser::Type type)
@@ -185,26 +194,26 @@ void PlaylistItem::guessTagInfo(TagGuesser::Type type)
     switch(type) {
     case TagGuesser::FileName:
     {
-	TagGuesser guesser(d->fileHandle.absFilePath());
-	Tag *tag = TagTransactionManager::duplicateTag(d->fileHandle.tag());
+        TagGuesser guesser(d->fileHandle.absFilePath());
+        Tag *tag = TagTransactionManager::duplicateTag(d->fileHandle.tag());
 
-	if(!guesser.title().isNull())
-	    tag->setTitle(guesser.title());
-	if(!guesser.artist().isNull())
-	    tag->setArtist(guesser.artist());
-	if(!guesser.album().isNull())
-	    tag->setAlbum(guesser.album());
-	if(!guesser.track().isNull())
-	    tag->setTrack(guesser.track().toInt());
-	if(!guesser.comment().isNull())
-	    tag->setComment(guesser.comment());
+        if(!guesser.title().isNull())
+            tag->setTitle(guesser.title());
+        if(!guesser.artist().isNull())
+            tag->setArtist(guesser.artist());
+        if(!guesser.album().isNull())
+            tag->setAlbum(guesser.album());
+        if(!guesser.track().isNull())
+            tag->setTrack(guesser.track().toInt());
+        if(!guesser.comment().isNull())
+            tag->setComment(guesser.comment());
 
-	TagTransactionManager::instance()->changeTagOnItem(this, tag);
-	break;
+        TagTransactionManager::instance()->changeTagOnItem(this, tag);
+        break;
     }
     case TagGuesser::MusicBrainz:
-	startMusicBrainzQuery(d->fileHandle);
-	break;
+        startMusicBrainzQuery(d->fileHandle);
+        break;
     }
 }
 
@@ -213,7 +222,7 @@ Playlist *PlaylistItem::playlist() const
     return static_cast<Playlist *>(listView());
 }
 
-QValueVector<int> PlaylistItem::cachedWidths() const
+QVector<int> PlaylistItem::cachedWidths() const
 {
     return d->cachedWidths;
 }
@@ -239,15 +248,15 @@ void PlaylistItem::clear()
 ////////////////////////////////////////////////////////////////////////////////
 
 PlaylistItem::PlaylistItem(CollectionListItem *item, Playlist *parent) :
-    KListViewItem(parent),
+    K3ListViewItem(parent),
     d(0),
     m_watched(0)
 {
     setup(item);
 }
 
-PlaylistItem::PlaylistItem(CollectionListItem *item, Playlist *parent, QListViewItem *after) :
-    KListViewItem(parent, after),
+PlaylistItem::PlaylistItem(CollectionListItem *item, Playlist *parent, Q3ListViewItem *after) :
+    K3ListViewItem(parent, after),
     d(0),
     m_watched(0)
 {
@@ -258,7 +267,7 @@ PlaylistItem::PlaylistItem(CollectionListItem *item, Playlist *parent, QListView
 // This constructor should only be used by the CollectionList subclass.
 
 PlaylistItem::PlaylistItem(CollectionList *parent) :
-    KListViewItem(parent),
+    K3ListViewItem(parent),
     m_watched(0)
 {
     d = new Data;
@@ -269,12 +278,12 @@ PlaylistItem::PlaylistItem(CollectionList *parent) :
 void PlaylistItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align)
 {
     if(!m_playingItems.contains(this))
-	return KListViewItem::paintCell(p, cg, column, width, align);
+        return K3ListViewItem::paintCell(p, cg, column, width, align);
 
-    QColorGroup colorGroup = cg;
+    QPalette colorGroup = cg;
 
-    QColor base = colorGroup.base();
-    QColor selection = colorGroup.highlight();
+    QColor base = colorGroup.color( QPalette::Base );
+    QColor selection = colorGroup.color( QPalette::Highlight );
 
     int r = (base.red() + selection.red()) / 2;
     int b = (base.blue() + selection.blue()) / 2;
@@ -282,44 +291,44 @@ void PlaylistItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int
 
     QColor c(r, g, b);
 
-    colorGroup.setColor(QColorGroup::Base, c);
-    QListViewItem::paintCell(p, colorGroup, column, width, align);
+    colorGroup.setColor(QPalette::Base, c);
+    Q3ListViewItem::paintCell(p, colorGroup, column, width, align);
 }
 
-int PlaylistItem::compare(QListViewItem *item, int column, bool ascending) const
+int PlaylistItem::compare(Q3ListViewItem *item, int column, bool ascending) const
 {
     // reimplemented from QListViewItem
 
     int offset = playlist()->columnOffset();
 
     if(!item)
-	return 0;
+        return 0;
 
     PlaylistItem *playlistItem = static_cast<PlaylistItem *>(item);
 
     // The following statments first check to see if you can sort based on the
     // specified column.  If the values for the two PlaylistItems are the same
-    // in that column it then trys to sort based on columns 1, 2, 3 and 0,
+    // in that column it then tries to sort based on columns 1, 2, 3 and 0,
     // (artist, album, track number, track name) in that order.
 
     int c = compare(this, playlistItem, column, ascending);
 
     if(c != 0)
-	return c;
+        return c;
     else {
-	// Loop through the columns doing comparisons until something is differnt.
-	// If all else is the same, compare the track name.
+        // Loop through the columns doing comparisons until something is differnt.
+        // If all else is the same, compare the track name.
 
-	int last = playlist()->isColumnVisible(AlbumColumn + offset) ? TrackNumberColumn : ArtistColumn;
+        int last = playlist()->isColumnVisible(AlbumColumn + offset) ? TrackNumberColumn : ArtistColumn;
 
-	for(int i = ArtistColumn; i <= last; i++) {
-	    if(playlist()->isColumnVisible(i + offset)) {
-		c = compare(this, playlistItem, i, ascending);
-		if(c != 0)
-		    return c;
-	    }
-	}
-	return compare(this, playlistItem, TrackColumn + offset, ascending);
+        for(int i = ArtistColumn; i <= last; i++) {
+            if(playlist()->isColumnVisible(i + offset)) {
+                c = compare(this, playlistItem, i, ascending);
+                if(c != 0)
+                    return c;
+            }
+        }
+        return compare(this, playlistItem, TrackColumn + offset, ascending);
     }
 }
 
@@ -328,12 +337,12 @@ int PlaylistItem::compare(const PlaylistItem *firstItem, const PlaylistItem *sec
     int offset = playlist()->columnOffset();
 
     if(column < 0 || column > lastColumn() + offset)
-	return 0;
+        return 0;
 
     if(column < offset) {
-	QString first = firstItem->text(column).lower();
-	QString second = secondItem->text(column).lower();
-	return first.localeAwareCompare(second);
+        QString first = firstItem->text(column).toLower();
+        QString second = secondItem->text(column).toLower();
+        return first.localeAwareCompare(second);
     }
 
     switch(column - offset) {
@@ -344,7 +353,7 @@ int PlaylistItem::compare(const PlaylistItem *firstItem, const PlaylistItem *sec
             return -1;
         else
             return 0;
-	break;
+        break;
     case LengthColumn:
         if(firstItem->d->fileHandle.tag()->seconds() > secondItem->d->fileHandle.tag()->seconds())
             return 1;
@@ -352,7 +361,7 @@ int PlaylistItem::compare(const PlaylistItem *firstItem, const PlaylistItem *sec
             return -1;
         else
             return 0;
-	break;
+        break;
     case BitrateColumn:
         if(firstItem->d->fileHandle.tag()->bitrate() > secondItem->d->fileHandle.tag()->bitrate())
             return 1;
@@ -360,7 +369,7 @@ int PlaylistItem::compare(const PlaylistItem *firstItem, const PlaylistItem *sec
             return -1;
         else
             return 0;
-	break;
+        break;
     case CoverColumn:
         if(firstItem->d->fileHandle.coverInfo()->hasCover() == secondItem->d->fileHandle.coverInfo()->hasCover())
             return 0;
@@ -370,8 +379,8 @@ int PlaylistItem::compare(const PlaylistItem *firstItem, const PlaylistItem *sec
             return 1;
         break;
     default:
-	return strcoll(firstItem->d->local8Bit[column - offset],
-		       secondItem->d->local8Bit[column - offset]);
+        return strcoll(firstItem->d->local8Bit[column - offset],
+                       secondItem->d->local8Bit[column - offset]);
     }
 }
 
@@ -397,14 +406,14 @@ void PlaylistItem::setup(CollectionListItem *item)
 // PlaylistItem::Pointer implementation
 ////////////////////////////////////////////////////////////////////////////////
 
-QMap<PlaylistItem *, QValueList<PlaylistItem::Pointer *> > PlaylistItem::Pointer::m_map; // static
+QMap<PlaylistItem *, QList<PlaylistItem::Pointer *> > PlaylistItem::Pointer::m_map; // static
 
 PlaylistItem::Pointer::Pointer(PlaylistItem *item) :
     m_item(item)
 {
     if(!m_item)
-	return;
-	
+        return;
+
     m_item->m_watched = true;
     m_map[m_item].append(this);
 }
@@ -418,31 +427,31 @@ PlaylistItem::Pointer::Pointer(const Pointer &p) :
 PlaylistItem::Pointer::~Pointer()
 {
     if(!m_item)
-	return;
+        return;
 
-    m_map[m_item].remove(this);
+    m_map[m_item].removeAll(this);
     if(m_map[m_item].isEmpty()) {
-	m_map.remove(m_item);
-	m_item->m_watched = false;
+        m_map.remove(m_item);
+        m_item->m_watched = false;
     }
 }
 
 PlaylistItem::Pointer &PlaylistItem::Pointer::operator=(PlaylistItem *item)
 {
     if(item == m_item)
-	return *this;
+        return *this;
 
     if(m_item) {
-	m_map[m_item].remove(this);
-	if(m_map[m_item].isEmpty()) {
-	    m_map.remove(m_item);
-	    m_item->m_watched = false;
-	}
+        m_map[m_item].removeAll(this);
+        if(m_map[m_item].isEmpty()) {
+            m_map.remove(m_item);
+            m_item->m_watched = false;
+        }
     }
 
     if(item) {
-	m_map[item].append(this);
-	item->m_watched = true;
+        m_map[item].append(this);
+        item->m_watched = true;
     }
 
     m_item = item;
@@ -453,11 +462,13 @@ PlaylistItem::Pointer &PlaylistItem::Pointer::operator=(PlaylistItem *item)
 void PlaylistItem::Pointer::clear(PlaylistItem *item) // static
 {
     if(!item)
-	return;
+        return;
 
-    QValueList<Pointer *> l = m_map[item];
-    for(QValueList<Pointer *>::Iterator it = l.begin(); it != l.end(); ++it)
-	(*it)->m_item = 0;
+    QList<Pointer *> l = m_map[item];
+    foreach(Pointer *pointer, l)
+        pointer->m_item = 0;
     m_map.remove(item);
     item->m_watched = false;
 }
+
+// vim: set et sw=4 tw=0 sta:
