@@ -1,11 +1,11 @@
 /*
  * Copyright (C) 2000 Rik Hemsley (rikkus) <rik@kde.org>
- * Copyright (C) 2000, 2001, 2002 Michael Matz <matz@kde.org>
+ * Copyright (C) 2000-2002 Michael Matz <matz@kde.org>
  * Copyright (C) 2001 Carsten Duvenhorst <duvenhorst@m2.uni-hannover.de>
  * Copyright (C) 2001 Adrian Schroeter <adrian@suse.de>
  * Copyright (C) 2003 Richard Lärkäng <richard@goteborg.utfors.se>
  * Copyright (C) 2003 Scott Wheeler <wheeler@kde.org>
- * Copyright (C) 2004, 2005 Benjamin Meyer <ben at meyerhome dot net>
+ * Copyright (C) 2004-2005 Benjamin Meyer <ben at meyerhome dot net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,18 +19,18 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
 
+#include <kdemacros.h>
 extern "C"
 {
 	#include <cdda_interface.h>
 	#include <cdda_paranoia.h>
 	void paranoiaCallback(long, int);
 
-	#include <kdemacros.h>
 	KDE_EXPORT int kdemain(int argc, char ** argv);
 }
 
@@ -44,53 +44,48 @@ extern "C"
 #include <sys/resource.h>
 
 #include <kmacroexpander.h>
-#include <qfile.h>
+#include <QFile>
 #include <qfileinfo.h>
 #include <kcmdlineargs.h>
 #include <kdebug.h>
 #include <kapplication.h>
 #include <klocale.h>
-#include <qregexp.h>
-
+#include <QRegExp>
+#include <QHash>
 // CDDB
 #include <client.h>
 #include "kcompactdisc.h"
 
 using namespace KIO;
-using namespace KCDDB;
 
-#define QFL1(x) QString::fromLatin1(x)
-#define DEFAULT_CD_DEVICE "/dev/cdrom"
 #define CDDB_INFORMATION "CDDB Information"
 
 using namespace AudioCD;
 
-static const KCmdLineOptions options[] =
-{
-    { "+protocol", I18N_NOOP("Protocol name"), 0 },
-    { "+pool", I18N_NOOP("Socket name"), 0 },
-    { "+app", I18N_NOOP("Socket name"), 0 },
-    KCmdLineLastOption
-};
-
 int kdemain(int argc, char ** argv)
 {
 	// KApplication uses libkcddb which needs a valid kapp pointer
 	// GUIenabled must be true as libkcddb sometimes wants to communicate
 	// with the user
 	putenv(strdup("SESSION_MANAGER="));
-	KApplication::disableAutoDcopRegistration();
-	KCmdLineArgs::init(argc, argv, "kio_audiocd", 0, 0, 0, 0);
-	KCmdLineArgs::addCmdLineOptions(options);
-	KApplication app(false, true);
+	//KApplication::disableAutoDcopRegistration();
+	KCmdLineArgs::init(argc, argv, "kio_audiocd", 0, KLocalizedString(), 0, KLocalizedString());
 
-	kdDebug(7117) << "Starting " << getpid() << endl;
+	KCmdLineOptions options;
+	options.add("+protocol", ki18n("Protocol name"));
+	options.add("+pool", ki18n("Socket name"));
+	options.add("+app", ki18n("Socket name"));
+	KCmdLineArgs::addCmdLineOptions(options);
+	KApplication app(true);
+
+	kDebug(7117) << "Starting " << getpid();
 
 	KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
-	AudioCDProtocol slave(args->arg(0), args->arg(1), args->arg(2));
+	AudioCDProtocol slave(args->arg(0).toLocal8Bit(), args->arg(1).toLocal8Bit(), args->arg(2).toLocal8Bit());
+	args->clear();
 	slave.dispatchLoop();
 
-	kdDebug(7117) << "Done" << endl;
+	kDebug(7117) << "Done";
 	return 0;
 }
 
@@ -108,6 +103,7 @@ public:
 		clearURLargs();
 		s_info = i18n("Information");
 		s_fullCD = i18n("Full CD");
+		discid = 0;
 	}
 
 	void clearURLargs() {
@@ -140,7 +136,7 @@ public:
 	KCompactDisc cd; // keep it around so that we don't assume the disk changed between every stat()
 
 	// CDDB items
-	KCDDB::CDDB::Result cddbResult;
+	KCDDB::Result cddbResult;
 	CDInfoList cddbList;
 	int cddbUserChoice; // URL settable
 	KCDDB::CDInfo cddbBestChoice;
@@ -158,7 +154,7 @@ public:
 
 int paranoia_read_limited_error;
 
-AudioCDProtocol::AudioCDProtocol(const QCString & protocol, const QCString & pool, const QCString & app)
+AudioCDProtocol::AudioCDProtocol(const QByteArray & protocol, const QByteArray & pool, const QByteArray & app)
 	: SlaveBase(protocol, pool, app)
 {
 	d = new Private;
@@ -167,21 +163,23 @@ AudioCDProtocol::AudioCDProtocol(const QCString & protocol, const QCString & poo
 	AudioCDEncoder::findAllPlugins(this, encoders);
 	encoderTypeCDA = encoderFromExtension(".cda");
 	encoderTypeWAV = encoderFromExtension(".wav");
-	encoders.setAutoDelete(true);
 }
 
 AudioCDProtocol::~AudioCDProtocol()
 {
-	delete d;
+    while (!encoders.isEmpty())
+        delete encoders.takeFirst();
+    delete d;
 }
 
 AudioCDEncoder *AudioCDProtocol::encoderFromExtension(const QString& extension)
 {
 	AudioCDEncoder *encoder;
-	for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
-		if(QString(".")+encoder->fileType() == extension)
-			return encoder;
-	}
+	for (int i = encoders.size()-1; i >= 0; --i) {
+            encoder = encoders.at(i);
+	    if(QString(".")+encoder->fileType() == extension)
+	        return encoder;
+    	}
 	Q_ASSERT(false);
 	return NULL;
 }
@@ -189,11 +187,11 @@ AudioCDEncoder *AudioCDProtocol::encoderFromExtension(const QString& extension)
 AudioCDEncoder *AudioCDProtocol::determineEncoder(const QString & filename)
 {
 	int len = filename.length();
-	int pos = filename.findRev('.');
+	int pos = filename.lastIndexOf('.');
 	return encoderFromExtension(filename.right(len - pos));
 }
 
-struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
+struct cdrom_drive * AudioCDProtocol::initRequest(const KUrl & url)
 {
 	if (url.hasHost())
 	{
@@ -213,17 +211,9 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		return 0;
 
 	// Update our knowledge of the disc
-
-#if defined(__linux__)
-       if(drive->ioctl_device_name && drive->ioctl_device_name[0])
-         d->cd.setDevice(drive->ioctl_device_name, 50, false);
-       else
-         d->cd.setDevice(drive->cdda_device_name, 50, false);
+#if defined(HAVE_CDDA_IOCTL_DEVICE)
+ 	d->cd.setDevice(drive->ioctl_device_name, 50, false);
 #else
-        d->cd.setDevice(drive->cdda_device_name, 50, false);
-#endif
-
-#if 0
 	// FreeBSD's cdparanoia as of january 5th 2006 has rather broken
 	// support for non-SCSI devices. Although it finds ATA cdroms just
 	// fine, there is no straightforward way to discover the device
@@ -235,12 +225,12 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		// For ATAPI devices, we have no real choice. Use the
 		// user selected value, even if there is none.
 		//
-		kdWarning(7117) << "Found an ATAPI device, assuming it is the one specified by the user." << endl;
+		kWarning(7117) << "Found an ATAPI device, assuming it is the one specified by the user.";
 		d->cd.setDevice( d->device );
 	}
 	else
 	{
-		kdDebug(7117) << "Found a SCSI or ATAPICAM device." << endl;
+		kDebug(7117) << "Found a SCSI or ATAPICAM device.";
 		if ( strlen(drive->dev->device_path) > 0 )
 		{
 			d->cd.setDevice( drive->dev->device_path );
@@ -254,13 +244,14 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 			QString devname = QString::fromLatin1( "/dev/%1%2" )
 				.arg( drive->dev->given_dev_name )
 				.arg( drive->dev->given_unit_number ) ;
-			kdDebug(7117) << "  Using derived name " << devname << endl;
+			kDebug(7117) << "  Using derived name " << devname;
 			d->cd.setDevice( devname );
 		}
 	}
 #endif
 
-	if (d->cd.discId() != d->discid && d->cd.discId() != d->cd.missingDisc){
+
+	if (d->cd.discId() != d->discid && !d->cd.isNoDisc()) {
 		d->discid = d->cd.discId();
 		d->tracks = d->cd.tracks();
 		for(uint i=0; i< d->cd.tracks(); i++)
@@ -268,30 +259,34 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 
 		KCDDB::Client c;
 		d->cddbResult = c.lookup(d->cd.discSignature());
-		d->cddbList = c.lookupResponse();
-		d->cddbBestChoice = c.bestLookupResponse();
+		if (d->cddbResult == Success)
+		{
+			d->cddbList = c.lookupResponse();
+			d->cddbBestChoice = d->cddbList.first();
+		}
 		generateTemplateTitles();
 	}
 
 	// Determine what file or folder that is wanted.
-	d->fname = url.fileName(false);
-	QString dname = url.directory(true, false);
+	d->fname = url.fileName(KUrl::ObeyTrailingSlash);
+	QString dname = url.directory(KUrl::AppendTrailingSlash);
 	if (!dname.isEmpty() && dname[0] == '/')
 		dname = dname.mid(1);
 
 	// Kong issue where they send dirs as files, double check
 	/* A hack, for when konqi wants to list the directory audiocd:/Bla
 		 it really submits this URL, instead of audiocd:/Bla/ to us. We could
-		 send (in listDir) the UDS_NAME as "Bla/" for directories, but then
+		 send (in listDir) the UDSEntry::UDS_NAME as "Bla/" for directories, but then
 		 konqi shows them as "Bla//" in the status line. */
 	// See if it is an encoder directory
 	AudioCDEncoder *encoder;
-	for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
-		if(encoder->type() == d->fname){
-			dname = d->fname;
-			d->fname = "";
-			break;
-		}
+	for (int i = encoders.size()-1; i >= 0; --i) {
+            encoder = encoders.at(i);
+	    if(encoder->type() == d->fname){
+		dname = d->fname;
+		d->fname = "";
+		break;
+	    }
 	}
 	// Other Hard coded directories
 	if (dname.isEmpty() && (d->fname == d->s_info || d->fname == d->s_fullCD ))
@@ -304,7 +299,8 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 
 	// See which directory they want
 	d->which_dir = Unknown;
-	for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
+	for (int i = encoders.size()-1; i >= 0; --i) {
+            encoder = encoders.at(i);
 		if(encoder->type() == dname){
 			d->which_dir = EncoderDir;
 			d->encoder_dir_type = encoder;
@@ -326,7 +322,7 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		QString name(d->fname);
 
 		// Remove extension
-		int dot = name.findRev('.');
+		int dot = name.lastIndexOf('.');
 		if (dot >= 0)
 			name.truncate(dot);
 
@@ -341,8 +337,8 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		else {
 			/* Not found in title list. Try hard to find a number in the
 				 string. */
-			unsigned int start = 0;
-			unsigned int end = 0;
+			int start = 0;
+			int end = 0;
 			// Find where the numbers start
 			while (start < name.length()){
 				if (name[start++].isDigit())
@@ -367,7 +363,7 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 	// Are we in the directory that lists "full CD" files?
 	d->req_allTracks = (dname.contains(d->s_fullCD));
 
-	kdDebug(7117) << "dir=" << dname << " file=" << d->fname
+	kDebug(7117) << "dir=" << dname << " file=" << d->fname
 		<< " req_track=" << d->req_track << " which_dir=" << d->which_dir << " full CD?=" << d->req_allTracks << endl;
 	return drive;
 }
@@ -391,7 +387,7 @@ bool AudioCDProtocol::getSectorsForRequest(struct cdrom_drive * drive, long & fi
 	return true;
 }
 
-void AudioCDProtocol::get(const KURL & url)
+void AudioCDProtocol::get(const KUrl & url)
 {
 	struct cdrom_drive * drive = initRequest(url);
 	if (!drive)
@@ -408,7 +404,7 @@ void AudioCDProtocol::get(const KURL & url)
 		for ( it = d->cddbList.begin(); it != d->cddbList.end(); ++it ){
 			if(count == choice){
 				mimeType("text/html");
-				data(QCString( (*it).toString().latin1() ));
+				data(QByteArray( (*it).toString().toLatin1() ));
 				// send an empty QByteArray to signal end of data.
 				data(QByteArray());
 				finished();
@@ -417,7 +413,7 @@ void AudioCDProtocol::get(const KURL & url)
 			}
 			count++;
 		}
-		if(!found && d->fname.contains(i18n(CDDB_INFORMATION)+":")){
+		if(!found && d->fname.contains(i18n(CDDB_INFORMATION)+':')){
 			mimeType("text/html");
 			//data(QCString( d->fname.latin1() ));
 			// send an empty QByteArray to signal end of data.
@@ -446,18 +442,18 @@ void AudioCDProtocol::get(const KURL & url)
 	}
 
 	KCDDB::CDInfo info;
-	if(d->cddbResult == KCDDB::CDDB::Success){
+	if(d->cddbResult == KCDDB::Success){
 		info = d->cddbBestChoice;
 
-		int track = d->req_track;
+		int track = d->req_track+1;
 
 		QString trackName;
 		// hack
 		// do we rip the whole CD?
 		if (d->req_allTracks){
-			track = 0;
+			track = 1;
 			// YES => the title of the file is the title of the CD
-			info.trackInfoList[track].title = info.title.utf8().data();
+			info.track(track-1).set(Title, info.get(Title));
 		}
 		encoder->fillSongInfo(info, track, "");
 	}
@@ -466,7 +462,7 @@ void AudioCDProtocol::get(const KURL & url)
 
 	unsigned long size = encoder->size(time_secs);
 	totalSize(size);
-	emit mimeType(QFL1(encoder->mimeType()));
+	emit mimeType(QLatin1String(encoder->mimeType()));
 
 	// Read data (track/disk) from the cd
 	paranoiaRead(drive, firstSector, lastSector, encoder, url.fileName(), size);
@@ -479,7 +475,7 @@ void AudioCDProtocol::get(const KURL & url)
 	finished();
 }
 
-void AudioCDProtocol::stat(const KURL & url)
+void AudioCDProtocol::stat(const KUrl & url)
 {
 	struct cdrom_drive * drive = initRequest(url);
 	if (!drive)
@@ -503,37 +499,28 @@ void AudioCDProtocol::stat(const KURL & url)
 
 	UDSEntry entry;
 
-	UDSAtom atom;
-	atom.m_uds = KIO::UDS_NAME;
-	atom.m_str = url.fileName().replace('/', QFL1("%2F"));
-	kdDebug(7117) << k_funcinfo << atom.m_str << endl;
-	entry.append(atom);
+	entry.insert( KIO::UDSEntry::UDS_NAME, url.fileName().replace('/', QLatin1String("%2F") ));
+
+	entry.insert( KIO::UDSEntry::UDS_FILE_TYPE,isFile ? S_IFREG : S_IFDIR);
 
-	atom.m_uds = KIO::UDS_FILE_TYPE;
-	atom.m_long = isFile ? S_IFREG : S_IFDIR;
-	entry.append(atom);
 
 	const mode_t _umask = ::umask(0);
 	::umask(_umask);
 
-	atom.m_uds = KIO::UDS_ACCESS;
-	atom.m_long = 0666 & (~_umask);
-	entry.append(atom);
+	entry.insert(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
 
-	atom.m_uds = KIO::UDS_SIZE;
 	if (!isFile)
 	{
-		atom.m_long = cdda_tracks(drive);
+		entry.insert( KIO::UDSEntry::UDS_SIZE, cdda_tracks(drive));
 	}
 	else
 	{
 			AudioCDEncoder *encoder = determineEncoder(d->fname);
 			long firstSector, lastSector;
 			getSectorsForRequest(drive, firstSector, lastSector);
-			atom.m_long = fileSize(firstSector, lastSector, encoder);
+			entry.insert( KIO::UDSEntry::UDS_SIZE,fileSize(firstSector, lastSector, encoder));
 	}
 
-	entry.append(atom);
 
 	statEntry(entry);
 
@@ -542,42 +529,26 @@ void AudioCDProtocol::stat(const KURL & url)
 	finished();
 }
 
-static void app_entry(UDSEntry& e, unsigned int uds, const QString& str)
-{
-	UDSAtom a;
-	a.m_uds = uds;
-	a.m_str = str;
-	e.append(a);
-}
-
-static void app_entry(UDSEntry& e, unsigned int uds, long l)
-{
-	UDSAtom a;
-	a.m_uds = uds;
-	a.m_long = l;
-	e.append(a);
-}
-
 static void app_dir(UDSEntry& e, const QString & n, size_t s)
 {
 	e.clear();
-	app_entry(e, KIO::UDS_NAME, QFile::decodeName(n.local8Bit()));
-	app_entry(e, KIO::UDS_FILE_TYPE, S_IFDIR);
-	app_entry(e, KIO::UDS_ACCESS, 0400);
-	app_entry(e, KIO::UDS_SIZE, s);
-	app_entry(e, KIO::UDS_MIME_TYPE, "inode/directory");
+	e.insert( KIO::UDSEntry::UDS_NAME, QFile::decodeName(n.toLocal8Bit()));
+	e.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+	e.insert( KIO::UDSEntry::UDS_ACCESS, 0400);
+	e.insert( KIO::UDSEntry::UDS_SIZE, s);
+	e.insert( KIO::UDSEntry::UDS_MIME_TYPE, QString("inode/directory"));
 }
 
 static void app_file(UDSEntry& e, const QString & n, size_t s)
 {
 	e.clear();
-	app_entry(e, KIO::UDS_NAME, QFile::decodeName(n.local8Bit()));
-	app_entry(e, KIO::UDS_FILE_TYPE, S_IFREG);
-	app_entry(e, KIO::UDS_ACCESS, 0400);
-	app_entry(e, KIO::UDS_SIZE, s);
+	e.insert( KIO::UDSEntry::UDS_NAME, QFile::decodeName(n.toLocal8Bit()));
+	e.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
+	e.insert( KIO::UDSEntry::UDS_ACCESS, 0400);
+	e.insert( KIO::UDSEntry::UDS_SIZE, s);
 }
 
-void AudioCDProtocol::listDir(const KURL & url)
+void AudioCDProtocol::listDir(const KUrl & url)
 {
 	struct cdrom_drive * drive = initRequest(url);
 
@@ -619,7 +590,7 @@ void AudioCDProtocol::listDir(const KURL & url)
 		}
 		// Error
 		if( count == 1 ) {
-			app_file(entry, QString("%1: %2.txt").arg(i18n(CDDB_INFORMATION)).arg(CDDB::resultToString(d->cddbResult)), ((*it).toString().length())+1);
+			app_file(entry, QString("%1: %2.txt").arg(i18n(CDDB_INFORMATION)).arg(KCDDB::resultToString(d->cddbResult)), ((*it).toString().length())+1);
 			count++;
 			listEntry(entry, false);
 		}
@@ -638,7 +609,8 @@ void AudioCDProtocol::listDir(const KURL & url)
 
 		// List the encoders
 		AudioCDEncoder *encoder;
-		for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
+		for (int i = encoders.size()-1; i >= 0; --i) {
+            		encoder = encoders.at(i);
 			// Skip the directory that is in the root (you can still go in it, just don't show it)
 			if( encoder == encoderTypeWAV )
 				continue;
@@ -652,8 +624,9 @@ void AudioCDProtocol::listDir(const KURL & url)
 		// if we're listing the "full CD" subdirectory :
 		if ( (d->which_dir == FullCD) ) {
 			AudioCDEncoder *encoder;
-			for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
-				if (d->cddbResult != KCDDB::CDDB::Success)
+			for (int i = encoders.size()-1; i >= 0; --i) {
+            			encoder = encoders.at(i);
+				if (d->cddbResult != KCDDB::Success)
 					addEntry(d->s_fullCD, encoder, drive, -1);
 				else
 					addEntry(d->templateAlbumName, encoder, drive, -1);
@@ -731,7 +704,7 @@ long AudioCDProtocol::fileSize(long firstSector, long lastSector, AudioCDEncoder
 
 struct cdrom_drive *AudioCDProtocol::getDrive()
 {
-	QCString device(QFile::encodeName(d->device));
+	QByteArray device(QFile::encodeName(d->device));
 
 	struct cdrom_drive * drive = 0;
 
@@ -743,19 +716,19 @@ struct cdrom_drive *AudioCDProtocol::getDrive()
 
 		if (0 == drive)
 		{
-			if (QFile(QFile::decodeName(DEFAULT_CD_DEVICE)).exists())
-				drive = cdda_identify(DEFAULT_CD_DEVICE, CDDA_MESSAGE_PRINTIT, 0);
+			if (QFile(QFile::decodeName(KCompactDisc::defaultCdromDeviceUrl().url().toAscii())).exists())
+				drive = cdda_identify(KCompactDisc::defaultCdromDeviceUrl().url().toAscii(), CDDA_MESSAGE_PRINTIT, 0);
 		}
 	}
 
 	if (0 == drive) {
-		kdDebug(7117) << "Can't find an audio CD on: \"" << d->device << "\"" << endl;
+		kDebug(7117) << "Can't find an audio CD on: \"" << d->device << "\"";
 
 		QFileInfo fi(d->device);
 		if(!fi.isReadable())
-			error(KIO::ERR_SLAVE_DEFINED, i18n("Device doesn't have read permissions for this account.  Check the read permissions on the device."));
+			error(KIO::ERR_SLAVE_DEFINED, i18n("Device does not have read permissions for this account.  Check the read permissions on the device."));
 		else if(!fi.isWritable())
-			error(KIO::ERR_SLAVE_DEFINED, i18n("Device doesn't have write permissions for this account.  Check the write permissions on the device."));
+			error(KIO::ERR_SLAVE_DEFINED, i18n("Device does not have write permissions for this account.  Check the write permissions on the device."));
 		else if(!fi.exists())
 			error(KIO::ERR_DOES_NOT_EXIST, d->device);
 		else error(KIO::ERR_SLAVE_DEFINED,
@@ -765,7 +738,7 @@ i18n("Unknown error.  If you have a cd in the drive try running cdparanoia -vsQ
 
 	if (0 != cdda_open(drive))
 	{
-		kdDebug(7117) << "cdda_open failed" << endl;
+		kDebug(7117) << "cdda_open failed";
 		error(KIO::ERR_CANNOT_OPEN_FOR_READING, d->device);
 		cdda_close(drive);
 		return 0;
@@ -788,7 +761,7 @@ void AudioCDProtocol::paranoiaRead(
 
 	cdrom_paranoia * paranoia = paranoia_init(drive);
 	if (0 == paranoia) {
-		kdDebug(7117) << "paranoia_init failed" << endl;
+		kDebug(7117) << "paranoia_init failed";
 		return;
 	}
 
@@ -837,9 +810,9 @@ void AudioCDProtocol::paranoiaRead(
 			warned = 1;
 		}
 		if (0 == buf) {
-			kdDebug(7117) << "Unrecoverable error in paranoia_read" << endl;
+			kDebug(7117) << "Unrecoverable error in paranoia_read";
 			ok = false;
-			error( ERR_SLAVE_DEFINED, i18n( "Error reading audio data for %1 from the CD" ).arg( fileName ) );
+			error( ERR_SLAVE_DEFINED, i18n( "Error reading audio data for %1 from the CD", fileName ) );
 			break;
 		}
 
@@ -847,12 +820,12 @@ void AudioCDProtocol::paranoiaRead(
 
 		int encoderProcessed = encoder->read(buf, CD_FRAMESAMPLES);
 		if(encoderProcessed == -1){
-			kdDebug(7117) << "Encoder processing error, stopping." << endl;
+			kDebug(7117) << "Encoder processing error, stopping.";
 			ok = false;
-			QString errMsg = i18n( "Couldn't read %1: encoding failed" ).arg( fileName );
+			QString errMsg = i18n( "Could not read %1: encoding failed", fileName );
 			QString details = encoder->lastErrorMessage();
 			if ( !details.isEmpty() )
-			    errMsg += "\n" + details;
+			    errMsg += '\n' + details;
 			error( ERR_SLAVE_DEFINED, errMsg );
 			break;
 		}
@@ -927,7 +900,7 @@ void AudioCDProtocol::paranoiaRead(
 		processedSize(processed);
 	}
 	else if ( ok ) // i.e. no error message already emitted
-		error( ERR_SLAVE_DEFINED, i18n( "Couldn't read %1: encoding failed" ).arg( fileName ) );
+		error( ERR_SLAVE_DEFINED, i18n( "Could not read %1: encoding failed", fileName ) );
 
 	paranoia_free(paranoia);
 	paranoia = 0;
@@ -937,44 +910,44 @@ void AudioCDProtocol::paranoiaRead(
  * Read the settings from the URL
  * @see loadSettings()
  */
-void AudioCDProtocol::parseURLArgs(const KURL & url)
+void AudioCDProtocol::parseURLArgs(const KUrl & url)
 {
 	d->clearURLargs();
 
-	QString query(KURL::decode_string(url.query()));
+	QString query(QUrl::fromPercentEncoding(url.query().toAscii()));
 
 	if (query.isEmpty() || query[0] != '?')
 		return;
 
 	query = query.mid(1); // Strip leading '?'.
 
-	QStringList tokens(QStringList::split('&', query));
+	QStringList tokens(query.split('&',QString::SkipEmptyParts));
 
 	for (QStringList::ConstIterator it(tokens.begin()); it != tokens.end(); ++it)
 	{
 		QString token(*it);
 
-		int equalsPos(token.find('='));
+		int equalsPos = token.indexOf('=');
 		if (-1 == equalsPos)
 			continue;
 
 		QString attribute(token.left(equalsPos));
 		QString value(token.mid(equalsPos + 1));
 
-		if (attribute == QFL1("device"))
+		if (attribute == QLatin1String("device"))
 			d->device = value;
-		else if (attribute == QFL1("paranoia_level"))
+		else if (attribute == QLatin1String("paranoia_level"))
 			d->paranoiaLevel = value.toInt();
-		else if (attribute == QFL1("fileNameTemplate"))
+		else if (attribute == QLatin1String("fileNameTemplate"))
 			d->fileNameTemplate = value;
-		else if (attribute == QFL1("albumNameTemplate"))
+		else if (attribute == QLatin1String("albumNameTemplate"))
 			d->albumTemplate = value;
-		else if (attribute == QFL1("cddbChoice"))
+		else if (attribute == QLatin1String("cddbChoice"))
 			d->cddbUserChoice = value.toInt();
-		else if (attribute == QFL1("niceLevel")){
+		else if (attribute == QLatin1String("niceLevel")){
 			int niceLevel = value.toInt();
 			if(setpriority(PRIO_PROCESS, getpid(), niceLevel) != 0)
-				kdDebug(7117) << "Setting nice level to (" << niceLevel << ") failed." << endl;
+				kDebug(7117) << "Setting nice level to (" << niceLevel << ") failed.";
 		}
 	}
 }
@@ -985,47 +958,46 @@ void AudioCDProtocol::parseURLArgs(const KURL & url)
  */
 void AudioCDProtocol::loadSettings()
 {
-	KConfig *config = new KConfig(QFL1("kcmaudiocdrc"));
+	KConfig *config = new KConfig(QLatin1String( "kcmaudiocdrc"), KConfig::NoGlobals );
+        KConfigGroup groupCDDA( config, "CDDA" );
 
-	config->setGroup(QFL1("CDDA"));
-
-	if (!config->readBoolEntry(QFL1("autosearch"),true)) {
-		d->device = config->readEntry(QFL1("device"),QFL1(DEFAULT_CD_DEVICE));
+	if (!groupCDDA.readEntry("autosearch", true)) {
+		d->device = groupCDDA.readEntry("device", QString(KCompactDisc::defaultCdromDeviceUrl().url()));
 	}
 
 	d->paranoiaLevel = 1; // enable paranoia error correction, but allow skipping
 
-	if (config->readBoolEntry("disable_paranoia",false)) {
+	if (groupCDDA.readEntry("disable_paranoia", false)) {
 		d->paranoiaLevel = 0; // disable all paranoia error correction
 	}
 
-	if (config->readBoolEntry("never_skip",true)) {
+	if (groupCDDA.readEntry("never_skip", true)) {
 		d->paranoiaLevel = 2;
 		// never skip on errors of the medium, should be default for high quality
 	}
 
-	d->reportErrors = config->readBoolEntry( "report_errors", false );
+	d->reportErrors = groupCDDA.readEntry( "report_errors", false );
 
-	if(config->hasKey("niceLevel")) {
-		int niceLevel = config->readNumEntry("niceLevel", 0);
+	if(groupCDDA.hasKey("niceLevel")) {
+		int niceLevel = groupCDDA.readEntry("niceLevel", 0);
 		if(setpriority(PRIO_PROCESS, getpid(), niceLevel) != 0)
-			kdDebug(7117) << "Setting nice level to (" << niceLevel << ") failed." << endl;
+			kDebug(7117) << "Setting nice level to (" << niceLevel << ") failed.";
 	}
 
 	// The default track filename template
-	config->setGroup("FileName");
-	d->fileNameTemplate = config->readEntry("file_name_template", "%{albumartist} - %{number} - %{title}");
-	d->albumTemplate = config->readEntry("album_template", "%{albumartist} - %{albumtitle}");
-	d->rsearch = config->readEntry("regexp_search");
-	d->rreplace = config->readEntry("regexp_replace");
+        KConfigGroup groupFileName( config, "FileName" );
+	d->fileNameTemplate = groupFileName.readEntry("file_name_template", "%{trackartist} - %{number} - %{title}");
+	d->albumTemplate = groupFileName.readEntry("album_template", "%{albumartist} - %{albumtitle}");
+	d->rsearch = groupFileName.readEntry("regexp_search");
+	d->rreplace = groupFileName.readEntry("regexp_replace");
 	// if the regular expressions are enclosed in qoutes. remove them
-	// otherwise it is not possible to search for a space " ", since an empty (only spaces) value is not 
+	// otherwise it is not possible to search for a space " ", since an empty (only spaces) value is not
 	// supported by KConfig, so the space has to be qouted, but then here the regexp searches really for " "
 	// instead of just the space. Alex
 	QRegExp qoutedString("^\".*\"$");
 	if (qoutedString.exactMatch(d->rsearch))
 	{
-        	d->rsearch=d->rsearch.mid(1, d->rsearch.length()-2);
+		d->rsearch=d->rsearch.mid(1, d->rsearch.length()-2);
 	}
 	if (qoutedString.exactMatch(d->rreplace))
 	{
@@ -1034,9 +1006,12 @@ void AudioCDProtocol::loadSettings()
 
 	// Tell the encoders to load their settings
 	AudioCDEncoder *encoder;
-	for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
-		encoder->init(); // ### TODO check return value!
-		encoder->loadSettings();
+	for (int i = encoders.size()-1; i >= 0; --i) {
+		encoder = encoders.at(i);
+		if (encoder->init())
+			encoder->loadSettings();
+		else
+			encoders.removeAt(i);
 	}
 
 	delete config;
@@ -1048,43 +1023,44 @@ void AudioCDProtocol::loadSettings()
 void AudioCDProtocol::generateTemplateTitles()
 {
 	d->templateTitles.clear();
-	if (d->cddbResult != KCDDB::CDDB::Success)
+	if (d->cddbResult != KCDDB::Success)
 	{
 		for (unsigned int i = 0; i < d->tracks; i++){
 			QString n;
-			d->templateTitles.append( i18n("Track %1").arg(n.sprintf("%02d", i + 1)));
+			d->templateTitles.append( i18n("Track %1", n.sprintf("%02d", i + 1)));
 		}
 		return;
 	}
 
 	KCDDB::CDInfo info = d->cddbBestChoice;
-	if(d->cddbUserChoice >= 0 && (((uint)d->cddbUserChoice) < d->cddbList.count()))
+	if(d->cddbUserChoice >= 0 && ((d->cddbUserChoice) < d->cddbList.count()))
 		info = d->cddbList[d->cddbUserChoice];
 
 	// Then generate the templates
 	d->templateTitles.clear();
 	for (uint i = 0; i < d->tracks; i++) {
-		QMap<QString, QString> macros;
-		macros["albumartist"] = info.artist;
-		macros["albumtitle"] = info.title;
-		macros["title"] = (info.trackInfoList[i].title);
+		QHash<QString, QString> macros;
+		macros["albumartist"] = info.get(Artist).toString();
+		macros["albumtitle"] = info.get(Title).toString();
+		macros["title"] = info.track(i).get(Title).toString();
+		macros["trackartist"] = info.track(i).get(Artist).toString();
 		QString n;
 		macros["number"] = n.sprintf("%02d", i + 1);
 		//macros["number"] = QString("%1").arg(i+1, 2, 10);
-		macros["genre"] = info.genre;
-		macros["year"] = QString::number(info.year);
+		macros["genre"] = info.get(Genre).toString();
+		macros["year"] = info.get(Year).toString();
 
-		QString title = KMacroExpander::expandMacros(d->fileNameTemplate, macros, '%').replace('/', QFL1("%2F"));
+		QString title = KMacroExpander::expandMacros(d->fileNameTemplate, macros, '%').replace('/', QLatin1String("%2F"));
 		title.replace( QRegExp(d->rsearch), d->rreplace );
 		d->templateTitles.append(title);
 	}
 
-	QMap<QString, QString> macros;
-	macros["albumartist"] = info.artist;
-	macros["albumtitle"] = info.title;
-	macros["genre"] = info.genre;
-	macros["year"] = QString::number(info.year);
-	d->templateAlbumName = KMacroExpander::expandMacros(d->albumTemplate, macros, '%').replace('/', QFL1("%2F"));
+	QHash<QString, QString> macros;
+	macros["albumartist"] = info.get(Artist).toString();
+	macros["albumtitle"] = info.get(Title).toString();
+	macros["genre"] = info.get(Genre).toString();
+	macros["year"] = info.get(Year).toString();
+	d->templateAlbumName = KMacroExpander::expandMacros(d->albumTemplate, macros, '%').replace('/', QLatin1String("%2F"));
 	d->templateAlbumName.replace( QRegExp(d->rsearch), d->rreplace );
 }
 
@@ -1098,54 +1074,54 @@ void paranoiaCallback(long, int function)
 {
 	switch(function){
 		case PARANOIA_CB_VERIFY:
-			//kdDebug(7117) << "PARANOIA_CB_VERIFY" << endl;
+			//kDebug(7117) << "PARANOIA_CB_VERIFY";
 			break;
 
 		case PARANOIA_CB_READ:
-			//kdDebug(7117) << "PARANOIA_CB_READ" << endl;
+			//kDebug(7117) << "PARANOIA_CB_READ";
 			break;
 
 		case PARANOIA_CB_FIXUP_EDGE:
-			//kdDebug(7117) << "PARANOIA_CB_FIXUP_EDGE" << endl;
+			//kDebug(7117) << "PARANOIA_CB_FIXUP_EDGE";
 			paranoia_read_limited_error = 2;
 			break;
 
 		case PARANOIA_CB_FIXUP_ATOM:
-			//kdDebug(7117) << "PARANOIA_CB_FIXUP_ATOM" << endl;
+			//kDebug(7117) << "PARANOIA_CB_FIXUP_ATOM";
 			paranoia_read_limited_error = 6;
 			break;
 
 		case PARANOIA_CB_READERR:
-			kdDebug(7117) << "PARANOIA_CB_READERR" << endl;
+			kDebug(7117) << "PARANOIA_CB_READERR";
 			paranoia_read_limited_error = 6;
 			break;
 
 		case PARANOIA_CB_SKIP:
-			kdDebug(7117) << "PARANOIA_CB_SKIP" << endl;
+			kDebug(7117) << "PARANOIA_CB_SKIP";
 			paranoia_read_limited_error = 8;
 			break;
 
 		case PARANOIA_CB_OVERLAP:
-			//kdDebug(7117) << "PARANOIA_CB_OVERLAP" << endl;
+			//kDebug(7117) << "PARANOIA_CB_OVERLAP";
 			break;
 
 		case PARANOIA_CB_SCRATCH:
-			kdDebug(7117) << "PARANOIA_CB_SCRATCH" << endl;
+			kDebug(7117) << "PARANOIA_CB_SCRATCH";
 			paranoia_read_limited_error = 7;
 			break;
 
 		case PARANOIA_CB_DRIFT:
-			//kdDebug(7117) << "PARANOIA_CB_DRIFT" << endl;
+			//kDebug(7117) << "PARANOIA_CB_DRIFT";
 			paranoia_read_limited_error = 4;
 			break;
 
 		case PARANOIA_CB_FIXUP_DROPPED:
-			kdDebug(7117) << "PARANOIA_CB_FIXUP_DROPPED" << endl;
+			kDebug(7117) << "PARANOIA_CB_FIXUP_DROPPED";
 			paranoia_read_limited_error = 5;
 			break;
 
 		case PARANOIA_CB_FIXUP_DUPED:
-			kdDebug(7117) << "PARANOIA_CB_FIXUP_DUPED" << endl;
+			kDebug(7117) << "PARANOIA_CB_FIXUP_DUPED";
 			paranoia_read_limited_error = 5;
 			break;
 	}
