@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <klocale.h>
@@ -24,20 +24,19 @@
 #include <kiconloader.h>
 #include <kconfig.h>
 #include <kaction.h>
-#include <kpopupmenu.h>
+#include <kmenu.h>
 #include <kglobalaccel.h>
-#include <kkeydialog.h>
+#include <kshortcutsdialog.h>
 #include <kdebug.h>
 
-#include <qobject.h>
 #include <qcursor.h>
 #include <qslider.h>
-#include <qlabel.h>
-#include <qlayout.h>
+#include <QMouseEvent>
 #include <qpixmap.h>
-#include <qtooltip.h>
 #include <qwmatrix.h>
 
+#include <kactioncollection.h>
+
 #include "mixer.h"
 #include "mixdevicewidget.h"
 #include "viewbase.h"
@@ -46,51 +45,41 @@
 #include "verticaltext.h"
 
 /**
- * Class that represents a single mix device, inlcuding PopUp, muteLED, ...
- * Used in KMix main window and DockWidget and PanelApplet.
- * It can be configured to include or exclude the recordLED and the muteLED.
+ * Base Class for any Widget that represents a MixDevice.
+ * The mix device can be a real (hardware bound) MixDevice or a virtual mix device.
+ *
  * The direction (horizontal, vertical) can be configured and whether it should
- * be "small"  (uses KSmallSlider instead of QSlider then).
+ * be "small"  (uses KSmallSlider instead of QSlider then). The actual implementations
+ * SHOULD honor these values - those who do not might not be suitable for placing in
+ * the panel applet or any other smallish settings.
  */
-MixDeviceWidget::MixDeviceWidget(Mixer *mixer, MixDevice* md,
+MixDeviceWidget::MixDeviceWidget(MixDevice* md,
                                  bool small, Qt::Orientation orientation,
-                                 QWidget* parent, ViewBase* mw, const char* name) :
-   QWidget( parent, name ), m_mixer(mixer), m_mixdevice( md ), m_mixerwidget( mw ),
+                                 QWidget* parent, ViewBase* mw) :
+   QWidget( parent ), m_mixdevice( md ), m_view( mw ),
    m_disabled( false ), _orientation( orientation ), m_small( small )
 {
    _mdwActions = new KActionCollection( this );
-   m_keys = new KGlobalAccel( this, "Keys" );
+   _mdwPopupActions = new KActionCollection( this );
 }
 
 MixDeviceWidget::~MixDeviceWidget()
 {
 }
 
-
 void MixDeviceWidget::addActionToPopup( KAction *action )
 {
-	_mdwActions->insert( action );
+   _mdwActions->addAction( action->objectName(), action );
 }
 
-
 bool MixDeviceWidget::isDisabled() const
 {
    return m_disabled;
 }
 
-
-KGlobalAccel *MixDeviceWidget::keys( void )
-{
-    return m_keys;
-}
-
 void MixDeviceWidget::defineKeys()
 {
-   if (m_keys) {
-      KKeyDialog::configure(m_keys, 0, false);
-      // The keys are saved in KMixerWidget::saveConfig, see kmixerwidget.cpp
-      m_keys->updateConnections();
-   }
+   KShortcutsDialog::configure(_mdwPopupActions ); // , KShortcutsEditor::LetterShortcutsAllowed, this);
 }
 
 void MixDeviceWidget::volumeChange( int ) { /* is virtual */ }
@@ -109,7 +98,7 @@ void MixDeviceWidget::setMutedColors( QColor , QColor , QColor ) { /* is virtual
 
 void MixDeviceWidget::mousePressEvent( QMouseEvent *e )
 {
-   if ( e->button()==RightButton )
+   if ( e->button() == Qt::RightButton )
       showContextMenu();
    else {
        QWidget::mousePressEvent(e);
