@@ -1,5 +1,5 @@
 
-/*  
+/*
     This file is part of the KDE libraries
     Copyright (C) 1997 Richard Moore (moorer@cs.man.ac.uk)
 
@@ -15,15 +15,15 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 
 #ifndef LED_LAMP_H
 #define LED_LAMP_H
 
-#include <qframe.h>
+#include <QFrame>
 
 
 class LedLamp : public QFrame
@@ -33,22 +33,22 @@ public:
 
   enum Type { Rect, Loop };
 
-  LedLamp(QWidget *parent=0, Type t=Rect);
+  explicit LedLamp(QWidget *parent=0, Type t=Rect);
 
   enum State { On, Off };
 
 
   State state() const { return s; }
-  
-
-  void setState(State state) { s= state; repaint(false); }
 
 
-  void toggleState() { if (s == On) s= Off; else if (s == Off) s= On; repaint(false); }
+  void setState(State state) { s= state; repaint(); }
+
+
+  void toggleState() { if (s == On) s= Off; else if (s == Off) s= On; repaint(); }
 public slots:
-  void toggle() { toggleState(); };
-  void on() { setState(On); };
-  void off() { setState(Off); };
+  void toggle() { toggleState(); }
+  void on() { setState(On); }
+  void off() { setState(Off); }
 protected:
   void drawContents(QPainter *);
 private:
